﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace GasolineTanker
{
    public partial class FormMapWithSetGasolineTanker : Form
    {
        private readonly Dictionary<string, AbstractMap> _mapsDict = new()
        {
            { "Simple map", new SimpleMap() },
            { "Long map", new LongMap() }
        };
        private MapWithSetGasolienTankerGeneric<DrawingObjectGasolineTanker, AbstractMap> _mapGasolineTankerCollectionGeneric;
        private readonly MapsCollection _mapsCollection;
        public FormMapWithSetGasolineTanker()
        {
            InitializeComponent();
            _mapsCollection = new MapsCollection(pictureBox.Width, pictureBox.Height);
            comboBoxSelectorMap.Items.Clear();
            foreach (var elem in _mapsDict)
            {
                comboBoxSelectorMap.Items.Add(elem.Key);
            }
        }
        private void ReloadMaps()
        {
            int index = listBoxMaps.SelectedIndex;
            listBoxMaps.Items.Clear();

            for (int i = 0; i < _mapsCollection.Keys.Count; i++)
            {
                listBoxMaps.Items.Add(_mapsCollection.Keys[i]);
            }

            if (listBoxMaps.Items.Count > 0 && (index == -1 || index >= listBoxMaps.Items.Count))
            {
                listBoxMaps.SelectedIndex = 0;
            }
            else if (listBoxMaps.Items.Count > 0 && index > -1 && index < listBoxMaps.Items.Count)
            {
                listBoxMaps.SelectedIndex = index;
            }
        }

        private void ButtonDeleteMap_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            if (MessageBox.Show($"Delete map {listBoxMaps.SelectedItem}?", "Delete", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                _mapsCollection.DelMap(listBoxMaps.SelectedItem?.ToString() ?? string.Empty);
                ReloadMaps();
            }
        }
        private void ButtonAddGasolineTanker_Click(object sender, EventArgs e)
        {
            {
                var formGasolineTankerConfig = new FormGasolineTankerConfig();
                formGasolineTankerConfig.AddEvent(AddGasolineTankerOnMap);
                formGasolineTankerConfig.Show();
            }
        }
        private void AddGasolineTankerOnMap(DrawingGasolineTanker drawingGasolineTanker)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            DrawingObjectGasolineTanker gasolineTanker = new(drawingGasolineTanker);
            if (_mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty] + gasolineTanker != -1)
            {
                MessageBox.Show("Object added");
                pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
            }
            else
            {
                MessageBox.Show("Failed to add object");
            }
        }

        private void ButtonShowStorage_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            pictureBox.Image = _mapGasolineTankerCollectionGeneric.ShowSet();
        }

        private void ButtonShowOnMap_Click(object sender, EventArgs e)
        {
            if(listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowOnMap();
        }
        private void ButtonMove_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            string name = ((Button)sender)?.Name ?? string.Empty;
            Direction dir = Direction.None;
            switch (name)
            {
                case "keyUp":
                    dir = Direction.Up;
                    break;
                case "keyDown":
                    dir = Direction.Down;
                    break;
                case "keyLeft":
                    dir = Direction.Left;
                    break;
                case "keyRight":
                    dir = Direction.Right;
                    break;
            }
            pictureBox.Image = _mapGasolineTankerCollectionGeneric.MoveObject(dir);
        }
        private void ComboBoxSelectorMap_SelectedIndexChanged(object sender, EventArgs e)
        {
            AbstractMap map = null;
            switch (comboBoxSelectorMap.Text)
            {
                case "Simple map":
                    map = new SimpleMap();
                    break;
                case "Long map":
                    map = new LongMap();
                    break;
            }
            if (map != null)
            {
                _mapGasolineTankerCollectionGeneric = new MapWithSetGasolienTankerGeneric<DrawingObjectGasolineTanker, AbstractMap>(
                    pictureBox.Width, pictureBox.Height, map);
            }
            else
            {
                _mapGasolineTankerCollectionGeneric = null;
            }
        }
        private void buttonAddMap_Click(object sender, EventArgs e)
        {
            if (comboBoxSelectorMap.SelectedIndex == -1 || string.IsNullOrEmpty(textBoxNewMapName.Text))
            {
                MessageBox.Show("Not all data saved", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (!_mapsDict.ContainsKey(comboBoxSelectorMap.Text))
            {
                MessageBox.Show("No such card", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _mapsCollection.AddMap(textBoxNewMapName.Text, _mapsDict[comboBoxSelectorMap.Text]);
            ReloadMaps();
        }
        private void buttonDeleteMap_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            if (MessageBox.Show($"Delete map {listBoxMaps.SelectedItem}?", "Delete", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                _mapsCollection.DelMap(listBoxMaps.SelectedItem?.ToString() ?? string.Empty);
                ReloadMaps();
            }
        }

        private void listBoxMaps_SelectedIndexChanged(object sender, EventArgs e)
        {
            pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
        }

        private void buttonRemoveGasolineTanker_Click(object sender, EventArgs e)
        {
            if (listBoxMaps.SelectedIndex == -1)
            {
                return;
            }
            if (string.IsNullOrEmpty(maskedTextBoxPosition.Text))
            {
                return;
            }
            if (MessageBox.Show("Delete object?", "Delete", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            int pos = Convert.ToInt32(maskedTextBoxPosition.Text);
            if (_mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty] - pos != null)
            {
                MessageBox.Show("Object delete");
                pictureBox.Image = _mapsCollection[listBoxMaps.SelectedItem?.ToString() ?? string.Empty].ShowSet();
            }
            else
            {
                MessageBox.Show("Errore delete object");
            }
        }

        private void SaveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                if (_mapsCollection.SaveData(saveFileDialog.FileName))
                {
                    MessageBox.Show("Save was successful", "Result", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                else
                {
                    MessageBox.Show("Not preserved", "Result", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void LoadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                if (_mapsCollection.LoadData(openFileDialog.FileName))
                {
                    MessageBox.Show("Download successful", "Result", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    ReloadMaps();
                }
                else
                {
                    MessageBox.Show("Didn't load", "Result", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }
    }
}
