﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace GasolineTanker
{
    internal class MapsCollection
    {
        readonly Dictionary<string, MapWithSetGasolienTankerGeneric<IDrawingObject, AbstractMap>> _mapStorages;

        public List<string> Keys => _mapStorages.Keys.ToList();

        private readonly int _pictureWidth;

        private readonly int _pictureHeight;
        private readonly char separatorDict = '|';
        private readonly char separatorData = ';';

        public MapsCollection(int pictureWidth, int pictureHeight)
        {
            _mapStorages = new Dictionary<string, MapWithSetGasolienTankerGeneric<IDrawingObject, AbstractMap>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
        }

        public void AddMap(string name, AbstractMap map)
        {
            if (!_mapStorages.ContainsKey(name)) _mapStorages.Add(name, new MapWithSetGasolienTankerGeneric<IDrawingObject, AbstractMap>(_pictureWidth, _pictureHeight, map));
        }

        public void DelMap(string name)
        {
            _mapStorages.Remove(name);
        }

        public MapWithSetGasolienTankerGeneric<IDrawingObject, AbstractMap> this[string ind]
        {
            get
            {
                if (_mapStorages.TryGetValue(ind, out var map)) return map;
                return null;
            }
        }
        public bool SaveData(string filename)
        {
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }
            using (StreamWriter sw = new StreamWriter(filename))
            {
                sw.WriteLine($"MapsCollection");
                foreach (var storage in _mapStorages)
                {
                    sw.WriteLine($"{storage.Key}{separatorDict}{storage.Value.GetData(separatorDict, separatorData)}");
                }
            }
            return true;
        }
        public bool LoadData(string filename)
        {
            if (!File.Exists(filename))
            {
                return false;
            }
            using (StreamReader sr = new StreamReader(filename))
            {
                string str;
                _mapStorages.Clear();
                str = sr.ReadLine();
                if (!str.Contains("MapsCollection"))
                {
                    return false;
                }
                while ((str = sr.ReadLine()) != null)
                {
                    var elem = str.Split(separatorDict);
                    AbstractMap map = null;
                    switch (elem[1])
                    {
                        case "Simple map":
                            map = new SimpleMap();
                            break;
                        case "Long map":
                            map = new LongMap();
                            break;
                    }
                    _mapStorages.Add(elem[0], new MapWithSetGasolienTankerGeneric<IDrawingObject, AbstractMap>(_pictureWidth, _pictureHeight, map));
                    _mapStorages[elem[0]].LoadData(elem[2].Split(separatorData, StringSplitOptions.RemoveEmptyEntries));
                }
                return true;
            }
        }
    }
}
